using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Collections;
using System.Drawing;

using WordCount;
using Rational;

/// <summary>
/// SessionData [Singleton, Facade]
/// 
/// This class maintens session data for PurifyPlusSample.
/// It also isolates implementation of the word counter from
/// the presenation level.
/// 
/// The PplusDemo.ASPX adds a single instance of this class 
/// to the session.
/// </summary>
public class SessionData
{
    // Application state
    private string m_strFileName;           // Uploaded file name
    private bool m_bFixPerformance = false; // Fix Performance check box
    private bool m_bFixLeaks = false;       // Fix Memory Leaks  check box
    private bool m_bUsePureApi = false;     // Use PureAPI check box
    private enSortBy m_enSortBy;            // Sorted field
    private enSortDir m_enSortDir;          // Sorting direction

    // WordCounter specific fields
    private WordCount.WordCount m_wc;                 // Word counter
    private Stream m_is;                    // Input stream (the source to parse)

    /// <summary>
    /// Constructor
    /// <remarks invoked implicitly/>
    /// </summary>
    public SessionData()
    {
        m_wc = new WordCount.WordCount();
        m_enSortBy = enSortBy.SORT_BY_WORD;
        m_enSortDir = enSortDir.SORT_ACCENDING;
    }

    /// <summary>
    /// FileName property
    /// </summary>
    public string FileName
    {
        get{ return m_strFileName; }
        set{ m_strFileName = value; }
    }

    /// <summary>
    /// Fix Performance property
    /// </summary>
    public bool bFixPerformance
    {
        set { m_bFixPerformance = value; }
    }

    /// <summary>
    /// Fix Leaks property
    /// </summary>
    public bool bFixLeaks
    {
        set { m_bFixLeaks = value; }
    }

    public bool bUsePureApi
    {
        set { m_bUsePureApi = value; }
    }

    /// <summary>
    /// Input Stream property
    /// </summary>
    public Stream InStream
    {
        set
        {
            if (m_is != null)
            {
                m_is.Close();
            }
            m_is = value;
        }
    }

    /// <summary>
    /// Sort By Words
    /// Instructs the parser to sort the result by words
    /// </summary>
    public void SortByWords()
    {
        if (m_enSortBy == enSortBy.SORT_BY_WORD)
        {
            m_enSortDir = m_enSortDir == enSortDir.SORT_ACCENDING
                        ? enSortDir.SORT_DESCENDING
                        : enSortDir.SORT_ACCENDING;
        }
        else
        {
            m_enSortBy = enSortBy.SORT_BY_WORD;
        }
    }

    /// <summary>
    /// Sort By Frequency
    /// Instructs the parser to sort the result by frequency
    /// </summary>
    public void SortByFreq()
    {
        if (m_enSortBy == enSortBy.SORT_BY_FREQ)
        {
            m_enSortDir = m_enSortDir == enSortDir.SORT_ACCENDING
                        ? enSortDir.SORT_DESCENDING
                        : enSortDir.SORT_ACCENDING;
        }
        else
        {
            m_enSortBy = enSortBy.SORT_BY_FREQ;
        }
    }

    /// <summary>
    /// Process Data
    /// Reads text from the input stream and creates
    /// hash table with words and counts
    /// <remarks>
    /// Using: 
    ///     m_is, m_bFixLeaks, m_bFixPerformance, m_wc
    /// </remarks>
    /// </summary>
    public void ProcessData()
    {
        if (m_is != null)
        {
            if (m_bUsePureApi)
            {
                PureAPI.ClearData();
                PureAPI.SaveData();
            }
            WordCount.WordCount.FixLeaks = m_bFixLeaks;
            WordCount.WordCount.FixPerformance = m_bFixPerformance;
            m_is.Position = 0;
            m_wc.Parse(m_is);
            if (m_bUsePureApi)
            {
                PureAPI.SaveData();
            }
        }
    }

    /// <summary>
    /// Show Data
    /// Puts the result data from m_wc into the result table (i_tab)
    /// </summary>
    /// <param name="i_tab">Result Table from ASPX page</param>
    public void ShowData(Table i_tab)
    {
        SortedResults sorted = new SortedResults(m_enSortBy , m_enSortDir);
        i_tab.Rows.Clear();

        // Generate header
        TableRow rowHead = new TableRow();
        TableCell cellHeadWord = new TableCell();
        TableCell cellHeadCount = new TableCell();
        rowHead.BackColor = Color.DarkGray;
        cellHeadWord.Text = "Words";
        cellHeadCount.Text = "Counts";
        if (m_enSortBy == enSortBy.SORT_BY_WORD)
            cellHeadWord.BackColor = Color.LightBlue;
        else
            cellHeadCount.BackColor = Color.LightBlue;
        rowHead.Cells.Add(cellHeadWord);
        rowHead.Cells.Add(cellHeadCount);
        i_tab.Rows.Add(rowHead);

        // Sort data
        foreach (DictionaryEntry entry in m_wc.m_Words)
        {
            string  strWord = (string)entry.Key;
            int nFreq = (int)(WordCount.WordContainer.IntBox)entry.Value;
            sorted.Add(strWord, nFreq);
        }
        // Populate the results table with sorted data
        foreach (ResultElement element in sorted.Keys)
        {

            TableRow row = new TableRow();
            TableCell cellWord = new TableCell();
            TableCell cellCount = new TableCell();
            cellWord.Text = element.strWord;
            cellCount.Text = element.nFreq.ToString();
            row.Cells.Add(cellWord);
            row.Cells.Add(cellCount);
            i_tab.Rows.Add(row);
        }

    }
}
