using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections;

namespace WordCount
{
    /// <summary>
    /// Enumeration: Sorting field.
    /// SORT_BY_WORD - Sort by word order
    /// SORT_BY_FREQ - Sort by Frequency
    /// </summary>
    public enum enSortBy { SORT_BY_WORD, SORT_BY_FREQ };
    /// <summary>
    /// Enumeration: Sorting order.
    /// SORT_ACCENDING  - Accending
    /// SORT_DESCENDING - Descending
    /// </summary>
    public enum enSortDir { SORT_ACCENDING, SORT_DESCENDING };

    /// <summary>
    /// Element of the result set
    /// </summary>
    public class ResultElement
    {
        private string  m_strWord;
        private int   m_nFreq;

        /// <summary>
        /// Property: strWord - Word
        /// </summary>
        public string strWord
        {
            get { return m_strWord; }
        }

        /// <summary>
        /// Property: nFreq - Frequency
        /// </summary>
        public int nFreq
        {
            get { return m_nFreq; }
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="i_strWord">Word</param>
        /// <param name="i_nrFreq">Frequency</param>
        public ResultElement(string i_strWord, Int32 i_nrFreq)
        {
            m_strWord = i_strWord;
            m_nFreq = i_nrFreq;
        }
    }

    /// <summary>
    /// Comparator for the result set.
    /// Changes sorting based on the selected criteria (field and order)
    /// </summary>
    public class ResultComparer : IComparer
    {
        private enSortBy m_enSortBy;
        private enSortDir m_enSortDir;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="i_enSortBy">Sorting field</param>
        /// <param name="i_enSortDir">Sorting order</param>
        public ResultComparer(enSortBy i_enSortBy, enSortDir i_enSortDir)
        {
            m_enSortBy = i_enSortBy;
            m_enSortDir = i_enSortDir;
        }

        /// <summary>
        /// Compare
        /// </summary>
        /// <param name="x">First result element</param>
        /// <param name="y">Second result element</param>
        /// <returns>0 if the elements are equal, 1 if x greater than y, -1 if x less than y</returns>
        public int Compare(ResultElement x, ResultElement y)
        {
            int result = 0;
            switch (m_enSortBy)
            {
                case enSortBy.SORT_BY_WORD:
                    result = x.strWord.CompareTo(y.strWord);
                    break;
                case enSortBy.SORT_BY_FREQ:
                    result = x.nFreq.CompareTo(y.nFreq);
                    if (result == 0)
                    {
                        result = x.strWord.CompareTo(y.strWord);
                    }
                    break;
            }
            if (m_enSortDir == enSortDir.SORT_DESCENDING)
            {
                result = -result;
            }
            return result;
        }

        #region IComparer Members
        public int Compare(Object x, Object y)
        {
            return Compare((ResultElement)x, (ResultElement)y);
        }
        #endregion
    }

    /// <summary>
    /// Sorted result set
    /// </summary>
    public class SortedResults : SortedList 
    {
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="i_enSortBy">Sorting field</param>
        /// <param name="i_enSortDir">Sorting order</param>
        public SortedResults(enSortBy i_enSortBy, enSortDir i_enSortDir)
            :base(new ResultComparer(i_enSortBy, i_enSortDir))
	    {
	    }

        /// <summary>
        /// Add element to the sorted result set.
        /// </summary>
        /// <param name="i_strWord"></param>
        /// <param name="i_nCount"></param>
        public void Add(string i_strWord, int i_nCount)
        {
            base.Add(new ResultElement(i_strWord, i_nCount), null);
        }
    }
}
