//*
//* stock.cpp
//*
//* | Licensed Materials - Use restricted, please refer to the "Samples" terms
//* | and conditions in the IBM International Program License Agreement (IPLA).  
//* (C) Copyright IBM Corporation. 1992, 2006. All Rights Reserved.
//*

// Stock.cpp : Defines the class behaviors for the application.
//
//#include <stdlib.h>
#include "stdafx.h"

#include "Stock.h"
#include "Stockdoc.h"
#include "mainfrm.h"
#include "Stockvw.h"
#include "SymView.h"
#include "SymDoc.h"
#include "SymDlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

// GetPriceStr - converts a price integer to a string. Assumes the
// lower two bits of PPrice are the fractional portion.
// Inputs:
//   PPrice - the scaled integer to convert
//   Pstr   - reference to the string to recieve the output
// Returns a pointer to the newly built string.
char* GetPriceStr (int PPrice, char* PStr)
{
	static char* cpFrac [4] = {"   ", "1/4", "1/2", "3/4"};
	sprintf (PStr, "%3.0i %s", PPrice >> 2, cpFrac [PPrice & 0x3]);
	return PStr;
}

CSymbolArray StockSymbols; //only instance declared 

// SYMSTRUCT is used to store symbol names, prices and price stability 
// (beta) information for use by the CSymArray class. It contains all 
// symbols that are displayed or traded in the stock simulation.
struct SYMSTRUCT {
	char* Name;
	int Price;
	int Beta;
	CSymDoc* Doc;
};
SYMSTRUCT StkSyms [] = {
	{"ADOB", 272, 3, NULL}, {"APPL", 152, 7, NULL}, {"ATDK", 141, 3, NULL}, 
	{"BORL",  59, 1, NULL}, {"CMPQ", 198, 3, NULL}, 
	{"CSCO", 336, 7, NULL}, {"DELL", 177, 1, NULL}, {"DGTL", 235, 3, NULL}, 
	{"EART", 136, 3, NULL},	{"GELE", 268, 3, NULL}, 
	{"GMGC", 43 , 3, NULL}, {"GPTA", 26 , 3, NULL}, {"IBM ", 386, 1, NULL}, 
	{"INFX", 112, 3, NULL}, {"INTL", 243, 7, NULL}, 
	{"MNGR",  81, 3, NULL}, {"MSFT", 349,15, NULL}, {"NOVL",  68, 1, NULL}, 
	{"ORCL", 182, 7, NULL}, {"PARC",  44, 7, NULL}, 
	{"PSFT", 168, 3, NULL}, {"PASW", 220, 1, NULL},	{"RATL", 150,15, NULL}, 
	{"RMDY", 246,15, NULL}, 
	{"SCO ",  25, 1, NULL}, {"SSTN",  54, 3, NULL},
	{"SLGR", 147, 7, NULL}, {"SUNM", 337, 1, NULL}, {"SYBS", 141, 3, NULL}, 
	{"SMTC", 106, 7, NULL}, {"3COM", 183, 3, NULL}, 
	{"3DO ",  45, 7, NULL}, {"XROX", 548, 0, NULL}, 
	{"", 0, NULL}};
#define MAXSYMBOL 0x1f

/////////////////////////////////////////////////////////////////////////////
// CSymbolArray

// CSymbolArray - this constructor calls restart to set the current indx.
CSymbolArray::CSymbolArray () : CObject () 
{	Restart (); }

//GenTransaction - generate a random stock transaction.
//
// Returns the index to the stock symbol that was changed
int CSymbolArray::GenTransaction ()
{
	int indx = rand () & MAXSYMBOL;	//pick symbol index
	StkSyms[indx].Price += GenDelta (indx);
	StkSyms[indx].Price = max (StkSyms[indx].Price, 1);
	if (StkSyms[indx].Doc) 
		StkSyms[indx].Doc->Transaction (StkSyms[indx].Price);
	return indx;
}

//GenDelta - generates the stock price change from the last trade
// Input: PIndex - the index number of the stock price to delta
// Returns the delta price for the stock number PIndex.
int CSymbolArray::GenDelta (int PIndex)
{
	int delta = rand () & StkSyms[PIndex].Beta;
	return (rand () & 1)? delta : -delta;
}	

//GetIndx - return index based on symbol name.
int CSymbolArray::GetIndx (char* PSymName)
{
	int indx;
	for (indx = 0; indx <= MAXSYMBOL; indx++)
		if (!strcmp (PSymName, StkSyms[indx].Name)) return indx;
	return -1;	//no match found
}

//GetName - find the symbol name for indx
char* CSymbolArray::GetName (int PIndx)	
{	return StkSyms[PIndx].Name; }

//GetPrice - returns the symbol price for indx
int CSymbolArray::GetPrice (int PIndx)
{	return StkSyms[PIndx].Price; }

//GetFirst - resets the internal index to the beginning.
void CSymbolArray::Restart ()
{	m_LastSymbol = 0; }

//GetNext - increments the internal index to point at the next symbol.
//Input: PDoc - if NULL then returns next symbol without a CSymDoc
//              attached to it. If a Stock portfolio pointer then 
//              returns next symbol contained within the portfolio.
//Returns pointer to the next symbols name string. NULL if no additional
//symbols are found.
char* CSymbolArray::GetNext ()
{
	for (++m_LastSymbol; m_LastSymbol < MAXSYMBOL; m_LastSymbol++)
		if (!StkSyms[m_LastSymbol].Doc)	
			return StkSyms[m_LastSymbol].Name;
	return NULL; 
}

char* CSymbolArray::GetNextAll ()
{	return (++m_LastSymbol <= MAXSYMBOL)? StkSyms[m_LastSymbol].Name: NULL; }

void CSymbolArray::SetDoc (int PIndx, CDocument* PDoc)
{
	StkSyms[PIndx].Doc = (CSymDoc*) PDoc;
}

/////////////////////////////////////////////////////////////////////////////
// CStockApp

BEGIN_MESSAGE_MAP(CStockApp, CWinApp)
	//{{AFX_MSG_MAP(CStockApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	ON_COMMAND(ID_VIEW_STOCKS, OnViewStocks)
	ON_COMMAND(ID_VIEW_ONE_STOCK, OnViewOneStock)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CStockApp construction

CStockApp::CStockApp()
{
    m_Ticker = (char *)HeapAlloc(GetProcessHeap(), 0, TICKER_SIZE);
    m_Ticker [TICKER_SIZE] = 0;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CStockApp object

CStockApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CStockApp initialization

BOOL CStockApp::InitInstance()
{
#ifndef VSNET_BUILD
	Enable3dControls();
#endif
	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// Register the application's document templates.
	m_pStockDocTemplate = new CMultiDocTemplate(
		IDR_STOCKTYPE,
		RUNTIME_CLASS(CStockDoc),
		RUNTIME_CLASS(CMDIChildWnd),          // standard MDI child frame
		RUNTIME_CLASS(CStockView) );
	AddDocTemplate(m_pStockDocTemplate);

	m_pSymDocTemplate = new CMultiDocTemplate(
		IDR_SYMTYPE,
		RUNTIME_CLASS(CSymDoc),
		RUNTIME_CLASS(CMDIChildWnd),          // standard MDI child frame
		RUNTIME_CLASS(CSymView) );
	AddDocTemplate(m_pSymDocTemplate);

	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

#ifdef NEVER
	// Don't register .stk files in the customers registry.
	// Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes();
#endif
	OnFileNew();	// create a new (empty) document

	pMainFrame->MDIGetActive()->SendMessage(WM_SYSCOMMAND, SC_MINIMIZE);

	// The main window has been initialized, so show and update it.
	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();

	m_wndStatusBar = &(pMainFrame->m_wndStatusBar);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

// Implementation
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CStockApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CStockApp commands

//CStockApp::OnFileNew - creates a new portfolio view.
void CStockApp::OnFileNew() 
{
	m_pStockDocTemplate->OpenDocumentFile (NULL, TRUE);	
}

void CStockApp::OnViewStocks() 
{
	m_pSymDocTemplate->OpenDocumentFile (NULL, TRUE);	
}

//CStockApp::OnViewOneStock - opens a dialog box to let the user
// select a stock to view then opens a new Symbol view.
void CStockApp::OnViewOneStock() 
{
	CSymDlg dlg;
	CSymDoc* doc;
	char *strSym;

	if (dlg.DoModal () == IDOK)
	{
		doc = (CSymDoc*) m_pSymDocTemplate->OpenDocumentFile (NULL, TRUE);
		if (strSym = dlg.m_Symbol.GetBuffer(4))	
			doc->InitSymbol (strSym);
	}
}

//CStockApp::MoveTicker - advances the ticker tape and redraws it on
// the status bar.
void CStockApp::MoveTicker() 
{
	m_TickerCounter++;
	if (m_TickerCounter >= SCROLL_OFFSET) 
	{
		::CopyMemory ((PVOID)m_Ticker, (const void *)(m_Ticker + SCROLL_OFFSET), NEW_SYM_START);

		m_TickerCounter = 0;
		// generate a random stock symbol transaction
		int indx = StockSymbols.GenTransaction ();
		sprintf (&(m_Ticker [NEW_SYM_START]), "...%s ", StockSymbols.GetName (indx));
		GetPriceStr (StockSymbols.GetPrice (indx), &(m_Ticker [NEW_PRC_START]));
	}
	else 
	{
		m_wndStatusBar->SetWindowText (m_Ticker + m_TickerCounter);
	}
}
