//
// Licensed Materials - Use restricted, please refer to the "Samples" terms and 
// conditions in the IBM International Program License Agreement (IPLA).  
//  Copyright IBM Corporation 1995, 2006. All Rights Reserved. 
//

using System;
using System.Threading;
using System.Collections;
using System.IO;
using System.Text;

namespace WordCount
{
	/// <summary>
	/// Rational Software 2002.
	/// Dot Net example used in the runtime analysis presentation.
	/// 
	/// Count words in the input stream.
	/// Use the Parse() method to set the input stream and start processsing
	/// The results will be stored in the public m_Words field.
	/// The WordContainer class used by m_Words derives from Hashtable
	/// and contains word-frequency pairs.
	/// </summary>
	/// 
	/// <example>
	/// 
	/// </example>
	/// 
	public class WordCount
	{
		private bool m_bFixPerformance; 
		/// <summary>Fix performance if true</summary>
		public bool bFixPerformance
		{
			get{return m_bFixPerformance;}
			set{m_bFixPerformance = value;}
		}

		private bool m_bFixLeaks;
		/// <summary>Fix memory leaks if true</summary>
		public bool bFixLeaks
		{
			get{return m_bFixLeaks;}
			set{m_bFixLeaks = value;}
		}

		// The parser plumbing. DFA matrix etc.
		 
		/// <summary>
		/// Character class. 
		/// We classify any input character to fit into one of the
		/// character classes below. This makes the DFA matrix much smaller
		/// </summary>
		enum CharClass_t
		{
			charEof     = 0, 			/// <summary>End of file</summary>
			charSpace   = 1,			/// <summary>White space</summary>
			charDelim   = 2,			/// <summary>Delimiter</summary>
			charValid   = 3,			/// <summary>Valid character</summary>
			charInvalid = 4,			/// <summary>Invalid character</summary>
			charLast    = 5			    /// <summary>Enum guard</summary>
		}

		/// <summary>
		/// Scan status.
		/// This status is used to swich DFA into a new state based on the
		/// previous status and the next input.
		/// </summary>
		enum ScanStat_t
		{
			statStart   = 0,			///<summary>DFA entry point</summary>
			statSpace   = 1,			///<summary>Inside WS</summary>
			statWord    = 2,			///<summary>Inside Word</summary>
			statEow     = 3,			///<summary>End of word reached</summary>
			statEof     = 4,			///<summary>End of file reached</summary>
			statError   = 5,			///<summary>Error</summary>
			statLast    = 6				///<summary>Enum guard</summary>
		}

		ScanStat_t[][] States = new ScanStat_t[][]
		{
		//				                   charEof              charSpace             charDelim             charValid             charInvalid            
		/* statStart */	new ScanStat_t[] {ScanStat_t.statError, ScanStat_t.statSpace, ScanStat_t.statEow,   ScanStat_t.statWord,  ScanStat_t.statError},
		/* statSpace */	new ScanStat_t[] {ScanStat_t.statEof  , ScanStat_t.statSpace, ScanStat_t.statSpace, ScanStat_t.statWord,  ScanStat_t.statError},  
		/* statWord  */	new ScanStat_t[] {ScanStat_t.statEof  , ScanStat_t.statEow,   ScanStat_t.statEow,   ScanStat_t.statWord,  ScanStat_t.statError},  
		/* StatEow 	 */ new ScanStat_t[] {ScanStat_t.statEof  , ScanStat_t.statSpace, ScanStat_t.statEow,   ScanStat_t.statWord,  ScanStat_t.statError},  
		/* StatEof 	 */ new ScanStat_t[] {ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError},  
		/* StatError */ new ScanStat_t[] {ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError},  
		};								  


		//----------------------------------------------------------------------
		/// <summary>
		/// Default constructor
		/// </summary>
		public WordCount()
		{
			statNext = ScanStat_t.statStart;
			m_Words = new WordContainer(this);
		}
		
		//----------------------------------------------------------------------
		/// <summary>
		/// <c>WordCount.Parse</c>
		/// Preconditions:
		///  i_stream not null and opened
		///
		/// This method is parsing the input stream into tokens and populates
		/// the word container with the value-frequency pairs.
		/// </summary>
		/// <param name="i_stream"> Input stream representing the source to parse</param>
		public void Parse(Stream i_stream)
		{
			m_stream = i_stream;
			Token tokenNext;
			// Let's pretend we forgot to clear the word container
			if(bFixLeaks)
				m_Words.Clear();
			// Initialization
			m_nWords = 0;
			m_nBytes = 0;
			statNext = ScanStat_t.statStart;
			// Parsing			
			tokenNext = Scan();
			while(tokenNext.GetToken() != Token.Token_t.tokEof)
			{
				switch(tokenNext.GetToken())
				{
					case Token.Token_t.tokWord:
						string strVal = tokenNext.GetValue();
						if(strVal != "")
						{	// Ignore empty strings;
							m_nWords++;
							m_Words.Set(strVal);
						}
						break;
					default:
						throw new WordCountException(tokenNext.GetValue());
				}
				tokenNext = Scan();
			}
		}

		//----------------------------------------------------------------------
		/// <summary>
		/// 
		/// <c>WordCount.Classify</c>
		///
		/// Classify character.
		/// input character is converted into character class (CharClass_t)
		/// Note that in this implementation char codes abov 128 are interpreted 
		/// as delimiters rather than error. This makes it possible to parse any file
		/// including binaries. This behaviour may be changed by returning
		/// CharClass_t.charError for certain characters.
		/// <seealso cref="CharClass_t"/>
		/// </summary>
		/// <param name="i_ch" type="int"> [in] character to classify </param>
		/// <returns></returns>
		/// 
		//
		private CharClass_t Classify(int i_ch)
		{
			char ch = (char)i_ch;
			CharClass_t ccRet;
			ccRet = CharClass_t.charInvalid;		// Assume the worst
			if(i_ch == 0xFFFF)						// End of file
				ccRet = CharClass_t.charEof;	
			else if(ch == ' ' || ch == '\t')		// White space
				ccRet = CharClass_t.charSpace;
			else if(ch == '_')						// Undescore is part of a word
				ccRet = CharClass_t.charValid;	
			else if((ch >= 'A' && ch <= 'Z')		// Vlaid chars
        		 ||	(ch >= 'a' && ch <= 'z')
				 || (ch >= '0' && ch <= '9')
				)
				ccRet = CharClass_t.charValid;	
			else if((ch > ' ' && ch < '0')			// Delimiters
				 || (ch == '\r' || ch == '\n')
				 || (ch >= ':' && ch <= '@')
				 || (ch > 'Z' && ch < 'a')
				 || (ch > 'z' && ch <= 128)
                 )
				ccRet = CharClass_t.charDelim;	
			else if(i_ch < ' ' || i_ch > 127)		// Make it configurable ?
				ccRet = CharClass_t.charDelim;		// May be considered error  or delim.
			return ccRet;
		}

		//----------------------------------------------------------------------
		/// <summary>
		/// WordCount.Scan
		/// Scan a next token from the input stream 
		/// </summary>
		/// <returns> Token - the next token parsed from the input stream </returns>
		///
		string strTok;									// Using string here is a bad idea
		StringBuilder sbTok = new StringBuilder();		// StringBuilder is for mutable strings

		private Token Scan()
		{
			int b;										// Next byte from the in stream
			Token.Token_t tokVal = Token.Token_t.tokError;
			CharClass_t charNext = CharClass_t.charLast;
			strTok = "";								// Reset token value
			sbTok.Remove(0, sbTok.Length);
			bool bDone = false;							// Asume we are not done yet

			while(!bDone)
			{
				// Get one char
				m_nBytes++;
				b = m_stream.ReadByte();
				charNext = Classify((char)b);
				// Fall through the DFA
				statNext = States[(int)statNext][(int)charNext];
				// Process the state
				switch(statNext)
				{
					// Space. Ignore
					case ScanStat_t.statSpace:
						break;			
					// Inside word. Keep processing
					case ScanStat_t.statWord:
						if(bFixPerformance)
							sbTok.Append((char)b);
						else
							strTok += (char)b;
						break;
					// End of word reached
					case ScanStat_t.statEow:
						if(bFixPerformance)
							strTok = sbTok.ToString();
						tokVal = Token.Token_t.tokWord;
						bDone = true;
						break;
					// End of file reached
					case ScanStat_t.statEof:
						strTok = "<<EOF>>";
						tokVal = Token.Token_t.tokEof;
						bDone = true;
						m_nBytes--;		//Don't count eof
						break;
					// statStart is invalid state so it's processed as error
					case ScanStat_t.statError:
					case ScanStat_t.statStart:
						strTok = "<<" + b.ToString() + ">>";
						tokVal = Token.Token_t.tokError;
						bDone = true;
						break;
				}
			} 
			// return the next token
			return new Token(tokVal, strTok);	
		}

		//----------------------------------------------------------------------
		/// <summary>
		/// WordCount.GetnWords
		/// 
		/// Get number of words currently parsed.
		/// </summary>
		/// <returns> ulong - number of words processed </returns>
		//
		public ulong GetnWords()
		{
			return m_nWords;
		}

		//----------------------------------------------------------------------
		/// <summary>
		/// WordCount.GetnBytes 
		/// </summary>
		/// <returns>ulong - number of bytes processed</returns>
		//
		public ulong GetnBytes()
		{
			return m_nBytes;
		}

		//+---------------------------------------------------------------------
		private Stream m_stream;		// Input stream 
		private ulong m_nWords = 0;		// Number of words currently processed
		private ulong m_nBytes = 0;		// Number of bytes currently processed
		private ScanStat_t statNext;	// Scanner status (DFA input)
		/// <summary>List/cache of words processed</summary>
		public WordContainer m_Words;	// List of the processed words with counts
	}

	//==========================================================================
	/// <summary>
	///  Class <c>Token</c> -  a token returned by the <c>Scan()</c> method 
	///  from the input stream. <seealso cref="WordCount.Scan"/>
	/// </summary>
	class Token
	{
		//----------------------------------------------------------------------
		public enum Token_t 
		{
			tokSpace,					/// <summary>White space</summary>
			tokWord,					/// <summary>Word</summary>
			tokDelim,					/// <summary>Delimiter</summary>
			tokError,					/// <summary>Error</summary>
			tokEof,						/// <summary>End of file</summary>
			tokLast						/// <summary>Guard</summary>
		} 

		//----------------------------------------------------------------------
		public Token(Token_t i_token, string i_strVal)
		{
			m_token = i_token;
			m_strVal = i_strVal;
		}

		//----------------------------------------------------------------------
		public Token_t GetToken()
		{
			return m_token;
		}

		//----------------------------------------------------------------------
		public string GetValue()
		{
			return m_strVal;
		}


		//+---------------------------------------------------------------------
		private Token_t m_token;
		private string	m_strVal;
	}
	
	//==========================================================================
	/// <summary>
	/// Word Container.
	/// In the 'slow' implementation represents a list of currently processed words
	/// thus the linear search is used to find out if the word was found before
	/// In the 'fast' implementation represent a cache of words.
	/// </summary>
	/// 
	public class WordContainer : Hashtable
	{
		private WordCount m_wc;
		//----------------------------------------------------------------------
		/// <summary>
		/// class IntBox implements integer unboxing thus making it possible 
		/// to change hash table entries.
		/// </summary>
		class IntBox
		{
			private int m_Val;
			public IntBox(int i_Val) { m_Val = i_Val; }
			public int get() { return m_Val; }
			public void set(int i_Val) { m_Val = i_Val; }
			public override string ToString() {return m_Val.ToString(); }
			public static implicit operator int (IntBox i_Val){ return i_Val.get(); }
		}
		//----------------------------------------------------------------------
		/// <summary>
		/// Constructor
		/// </summary>
		public WordContainer(WordCount i_wc)
		{
			m_wc = i_wc;
		}

		//----------------------------------------------------------------------
		/// <summary>
		/// Set the next word value extracted from the input stream.
		/// If the word already in the list the frequency counter will be
		/// incremented otherwise the new word is added to the list with the 
		/// counter = 1
		/// </summary>
		/// <param name="i_strKey"> next word from the in stream </param>
		public void Set(string i_strKey)
		{
			IntBox Val = Find(i_strKey);
			if(null != Val)
			{	// word found. Bump the counter
				Val.set(Val.get() + 1);
			}
			else
			{	// New word. Remember it.
				Add(i_strKey, new IntBox(1));
			}
		}

		//----------------------------------------------------------------------
		/// <summary>
		/// Find a word in the list.
		/// </summary>
		/// <param name="i_strKey"></param>
		/// <returns></returns>
		IntBox Find(string i_strKey)
		{
			if(!m_wc.bFixPerformance)
			{	// Bad searching algorithm:
				// Using sequential search in the list of words
				// slows down the processing.
				IDictionaryEnumerator entry = GetEnumerator();
				while(entry.MoveNext())
				{
					if(i_strKey == entry.Key.ToString())
					{
						return (IntBox)entry.Value;
					}
				}
			}
			else
			{	// Good searching algorithm
				// Key lookup in the cache table works 10 times faster!
				return (IntBox)this[i_strKey];
			}
			return null;
		}
	}

	//==========================================================================
	/// <summary>
	/// Word Counter exception class
	/// </summary>
	/// <exception cref=" Exception"> 
	/// Thrown if something goes wrong with the word processing
	/// </exception>
	public class WordCountException : Exception
	{
		/// <summary>
		/// This exception is thrown in case anything goes wrong while
		/// counting words.
		/// </summary>
		/// <param name="i_str"> Message assigned to the exception </param>
		public WordCountException(string i_str)
			: base("Parsing error:" + i_str)
		{
		}
	}

}
//= End of WordCount.cs ========================================================
